"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("hornet-js-utils/src/index");
var Promise = require("bluebird");
Promise.config({ cancellation: true });
var CancellablePromise = /** @class */ (function () {
    function CancellablePromise() {
    }
    /**
     * Retourne une promesse pouvant etre annulée
     * en même temps que les autres promesses annulables
     * elle sera enregistrée dans la variable "cancellablePromises" du cls
     * aves les autres promesses annulables
     * @param {any} executor
     */
    CancellablePromise.getPromise = function (executor) {
        var promise = new Promise(executor);
        this.registerPromiseInCls(promise);
        return promise;
    };
    /**
     * Retourne une promesse pouvant etre annulée à n'importe quel moment
     * elle sera enregistrée dans la variable "cancellablePromise[name]" du cls
     * @param {any} executor
     * @param {string} name nom de la promesse
     */
    CancellablePromise.getNamedPromise = function (executor, name) {
        var _this = this;
        var promise = new Promise(executor);
        this.registerPromiseInClsWithName(promise, name);
        return promise.then((function (result) {
            _this.unregisterPromiseInClsWithName(name);
            return result;
        }).bind(this, promise)).catch((function (error) {
            _this.unregisterPromiseInClsWithName(name);
            return error;
        }).bind(this, promise));
    };
    /**
     * Enregistre la promesse dans le Cls
     * @param {Promise<any>} promise promesse à enregistrer
     */
    CancellablePromise.registerPromiseInCls = function (promise) {
        var promises = index_1.Utils.getCls(CancellablePromise.CANCELLABLE_PROMISES) || [];
        if (promises.indexOf(promise) < 0) {
            promises.push(promise);
            index_1.Utils.setCls(CancellablePromise.CANCELLABLE_PROMISES, promises);
        }
    };
    /**
     * Enregistre la promesse dans le Cls à l'aide de son nom
     * @param {Promise<any>} promise promesse à enregistrer
     * @param {string} name nom de la promesse
     */
    CancellablePromise.registerPromiseInClsWithName = function (promise, name) {
        var promiseCls = index_1.Utils.getCls(CancellablePromise.CANCELLABLE_PROMISE);
        if (!promiseCls) {
            promiseCls = {};
        }
        promiseCls[name] = promise;
        index_1.Utils.setCls(CancellablePromise.CANCELLABLE_PROMISE, promiseCls);
    };
    /**
     * Desenregistre la promesse dans le Cls à l'aide de son nom
     * @param {string} name nom de la promesse
     */
    CancellablePromise.unregisterPromiseInClsWithName = function (name) {
        var promises = index_1.Utils.getCls(CancellablePromise.CANCELLABLE_PROMISE);
        if (promises) {
            var promiseCls = promises[name];
            if (promiseCls) {
                promises[name] = null;
                index_1.Utils.setCls(CancellablePromise.CANCELLABLE_PROMISE, promises);
            }
        }
    };
    /**
     * Cancel et désenregistre la promesse dans le Cls à l'aide de son nom
     * @param {string} name nom de la promesse
     */
    CancellablePromise.cancelPromiseWithName = function (name) {
        var promises = index_1.Utils.getCls(CancellablePromise.CANCELLABLE_PROMISE);
        if (promises) {
            var promiseCls = promises[name];
            if (promiseCls) {
                promiseCls.cancel();
                this.unregisterPromiseInClsWithName(name);
            }
        }
    };
    /**
     * Cancel et désenregistre les promesses présentes dans le Cls
     * @param includeNamed inclure les promesses nommées
     */
    CancellablePromise.clearAllCancellablePromises = function (includeNamed) {
        if (includeNamed === void 0) { includeNamed = false; }
        CancellablePromise.clearAllUnamedCancellablePromises();
        if (includeNamed) {
            CancellablePromise.clearAllNamedPromises();
        }
    };
    /**
     * Cancel et désenregistre les promesses présentes dans le Cls
     * Ces promesses ne sont pas nommées
     */
    CancellablePromise.clearAllUnamedCancellablePromises = function () {
        var promises = index_1.Utils.getCls(CancellablePromise.CANCELLABLE_PROMISES);
        index_1.Utils.setCls(CancellablePromise.CANCELLABLE_PROMISES, []);
        if (promises) {
            promises.forEach(function (promise) {
                if (promise.cancel) {
                    promise.cancel();
                }
            });
        }
    };
    /**
     * Cancel et désenregistre les promesses présentes dans le Cls
     * Ces promesses sont nommées
     */
    CancellablePromise.clearAllNamedPromises = function () {
        var _this = this;
        var namedPromises = index_1.Utils.getCls(CancellablePromise.CANCELLABLE_PROMISE);
        if (namedPromises) {
            Object.keys(namedPromises).forEach((function (key) {
                _this.cancelPromiseWithName(key);
            }).bind(this));
        }
    };
    CancellablePromise.CANCELLABLE_PROMISE = "cancellablePromise";
    CancellablePromise.CANCELLABLE_PROMISES = "cancellablePromises";
    return CancellablePromise;
}());
exports.CancellablePromise = CancellablePromise;

//# sourceMappingURL=cancellable-promise.js.map
